package	jp.co.sds_corp.util.daemon.manager;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.IInitializer;
import jp.co.sds_corp.util.daemon.IService;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* デーモンなしでサービスを起動します。
<P>
&lt;Initialize&gt;要素には、起動時に（デーモンの起動前に）呼び出す
{@link IInitializer}の情報を記述します。
<P>
&lt;Service&gt;要素には、起動するサービスの情報を記述します。
*/
public class ServiceManager
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	/** 初期処理 を指定するXML要素名 */
	protected String KEY_INITIALIZER = "Initialize";
	/** Service を指定するXML要素名 */
	protected String KEY_SERVICE = "Service";
	private Map<String,IInitializer>	initMap = new LinkedHashMap<String,IInitializer>();
	private Map<String,IService>	serviceMap = new LinkedHashMap<String,IService>();
	private boolean bExit = false;

	protected ServiceManager() {
		logger.debug( this, "start:" + getClass().getName() );
		Thread shut = new Thread() {
			public void run() {
				onShutdown();
			}
		};
		Runtime.getRuntime().addShutdownHook( shut );
	}

	public void init() throws Exception {
		String config = "service.xml";
		if ( System.getProperty( "ServiceManager.config" ) != null ) {
			config = System.getProperty( "ServiceManager.config" );
		}
		init( new DOMConfiguration( config ));
	}

	public void init( IConfiguration config ) throws Exception {
		Iterator<IConfiguration> itr;
		itr = config.getChilds( KEY_INITIALIZER );
		initMap.putAll( createAllInitializer( itr ) );

		Iterator<String> itr2 = initMap.keySet().iterator();
		while ( itr2.hasNext()) {
			IInitializer init = (IInitializer )initMap.get( itr2.next());
			init.start();
		}

		itr = config.getChilds( KEY_SERVICE );
		serviceMap.putAll( createAllService( itr ) );
		logger.info( this, "ServiceManager initialized." );
	}

	/**
	* 指定された初期化処理({@link IInitializer})を全て生成して呼び出します。
	*/
	protected Map<String,IInitializer> createAllInitializer( Iterator<IConfiguration> itr )
		throws Exception
	{
		Map<String,IInitializer> ret = new LinkedHashMap<String,IInitializer>();
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			IInitializer init = createInitializer( c );
			ret.put( c.getAttributes().get( "name" ), init );
		}
		return ret;
	}

	/**
	* 指定された初期化処理({@link IInitializer})を生成して呼び出します。
	*/
	protected IInitializer createInitializer( IConfiguration config )
		throws Exception
	{
		Object iObj = ConfigUtility.createInstance( config );
		IInitializer init = (IInitializer)iObj;
		logger.debug( this, "loading Initializer: " + init.getClass().getName());
		try {
			init.init( config );
		} catch ( Exception ex ) {
			logger.error( this, "Initializer [" + init + "] init error. " );
			throw( ex );
		}
		return init;
	}

	/**
	* 指定されたサービスを全て生成して呼び出します。
	*/
	protected Map<String,IService> createAllService( Iterator<IConfiguration> itr )
		throws Exception
	{
		Map<String,IService> ret = new LinkedHashMap<String,IService>();
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			IService svc = createService( c );
			ret.put( c.getAttributes().get( "name" ), svc );
		}
		return ret;
	}

	/**
	* 指定されたサービスを生成します。
	*/
	protected IService createService( IConfiguration config )
		throws Exception
	{
		Object iObj = ConfigUtility.createInstance( config );
		IService svc = (IService )iObj;
		logger.debug( this, "loading Service: " + svc.getClass().getName());
		try {
			svc.init( config );
		} catch ( Exception ex ) {
			logger.error( this, "Service [" + svc + "] init error. " );
			throw( ex );
		}
		return svc;
	}

	public Map<String,IService> getServiceMap() {
		return serviceMap;
	}

	public int start() throws Exception {
		int nRet = -1;
		Iterator<String> itr;
		itr = serviceMap.keySet().iterator();
		while ( itr.hasNext()) {
			IService svc = (IService )serviceMap.get( itr.next());
			logger.debug( this, "start: " + svc.getName() + ":"
								+ svc.getClass().getName() );
			svc.start();
		}
		return nRet;
	}

	public void stop() {
		Iterator itr;
		itr = serviceMap.keySet().iterator();
		while ( itr.hasNext()) {
			IService svc = (IService )serviceMap.get( itr.next());
			logger.debug( this, "stopping " + svc.getName() );
			svc.stop();
			logger.debug( this, "stopping " + svc.getName() );
		}
		itr = serviceMap.keySet().iterator();
		while ( itr.hasNext()) {
			IInitializer init = (IInitializer )initMap.get( itr.next());
			init.stop();
		}
		logger.debug( this, "stopped " + getClass().getName() );
		setExit( true );
	}

	/**
	* 終了時の処理を行ないます。
	*/
	public void onShutdown() {
		if ( !isExit() ) {
			stop();
		}
	}

	protected synchronized void setExit( boolean var ) {
		bExit = var;
	}
	protected synchronized boolean isExit() {
		return bExit;
	}

	public static void main( String[] args ) {
		ServiceManager obj = new ServiceManager();
		int nRet = 9;
		try {
			obj.init();
			nRet = obj.start();
			obj.stop();
		} catch ( Exception ex ) {
			logger.error( ServiceManager.class, "error", ex );
		}
		System.exit( nRet );
	}

}

