package	jp.co.sds_corp.util.daemon.remote.socket;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.HashMap;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.remote.IRemoteClient;

/**
* デーモンのSocket I/Fのクライアントです。
* DaemonManagerが使用するXMLファイルから、ソケットのポートを取得し
* DaemonManagerにコマンドを送信します。
*/
public class SocketClient implements IRemoteClient
{
	String	host;	// ホスト名
	int	port;	// ポート番号
	int	timeout;	// タイムアウト
	boolean	connected = false;	// 接続しているか

	/**
	* クライアントのタイプを返します。
	*/
	public String getType() {
		return "Socket";
	}

	/**
	* デーモンマネージャの ソケットクライアントを初期化します。
	* @param    config  設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		host = config.get( "host", null );
		port = Integer.valueOf( config.get( "port" )).intValue();
		timeout = Integer.valueOf( config.get( "timeout", "5000" )).intValue();
	}

	/**
	* 接続を開始します。
	*/
	public void connect() throws Exception {
		connected = true;
	}
	/**
	* 接続を終了します。
	*/
	public void disconnect() throws Exception {
		connected = false;
	}

	/**
	* 接続しているかを設定します。
	* @param	var		設定するデータ
	*/
	public void setConnected( boolean var ) {
		this.connected = var;
	}
	/**
	* 接続しているかを取得します。
	* @return	接続しているか
	*/
	public boolean isConnected() {
		return connected;
	}

	/**
	* リモートのメソッドを呼び出します。
	* @param    cmd     メソッド名
	* @param    cls     パラメータのクラス
	* @param    prm     パラメータのオブジェクト
	*/
	public Object execRemote( String cmd, Class<?>[] cls, Object[] prm )
		throws Exception
	{
		Receiver rcv = new Receiver( cmd, cls, prm );
		Thread thr = new Thread( rcv );
		thr.setName( "SocketClient:Receiver" );
		thr.start();
		try {
			thr.join( timeout );
		} catch ( InterruptedException ex ) {
		}
		if ( thr.isAlive()) {
			thr.interrupt();
			Thread.yield();
			throw( new SocketTimeoutException( "no response" ));
		}
		if ( rcv.ex != null ) {
			throw( rcv.ex );
		}
		return rcv.res;
	}

	/**
	* リモートのメソッドを呼び出します。
	* @param    cmd     メソッド名
	*/
	public Object execRemote( String cmd )
		throws Exception
	{
		Object ret = execRemote( cmd, null, null );
		return ret;
	}

	private class Receiver implements Runnable {
		Object res;
		Exception ex;
		String cmd;
		Class<?>[] cls;
		Object[] prm;
		Receiver( String cmd, Class<?>[] cls, Object[] prm ) {
			this.cmd = cmd;
			this.cls = cls;
			this.prm = prm;
		}
		public void run() {
			Socket sock = null;
			try {
				HashMap<String,Object> reqMap = new HashMap<String,Object>();
				reqMap.put( "command", cmd );
				if ( cls != null ) {
					reqMap.put( "class", cls );
				}
				if ( prm != null ) {
					reqMap.put( "param", prm );
				}
				sock = new Socket( host, port );
				ObjectOutputStream os = new ObjectOutputStream(
					new BufferedOutputStream( sock.getOutputStream() ));
				os.writeObject( reqMap );
				os.flush();

				ObjectInputStream is = new ObjectInputStream(
					new BufferedInputStream( sock.getInputStream() ));
				res = is.readObject();
				os.close();
				is.close();
			} catch ( Exception ex ) {
				this.ex = ex;
			} finally {
				try {
					sock.close();
				} catch ( Exception ex ) {
				}
			}
		}
	}

	/**
	* ソケットクライアントを終了します。
	*/
	public void exit() {
	}

	/**
	* ホスト名を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var;
	}
	/**
	* ホスト名を取得します。
	* @return	ホスト名
	*/
	public String getHost() {
		return host;
	}

	/**
	* ポート番号を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var;
	}
	/**
	* ポート番号を取得します。
	* @return	ポート番号
	*/
	public int getPort() {
		return port;
	}

	/**
	* タイムアウトを設定します。
	* @param	var		設定するデータ
	*/
	public void setTimeout( int var ) {
		this.timeout = var;
	}
	/**
	* タイムアウトを取得します。
	* @return	タイムアウト
	*/
	public int getTimeout() {
		return timeout;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		config.setAttribute( "type", "Socket" );
		config.addChild( "host", host );
		config.addChild( "port", String.valueOf( port ));
		config.addChild( "timeout", String.valueOf( timeout ));
	}

	/**
	* このクライアントの文字列表現を返します。
	* @return	クライアントの文字列表現
	*/
	public String toString() {
		String ret = "SocketClient [" + host + ":" + port + "]"
						+ " timeout=" + timeout;
		return ret;
	}

}
